@@ -1,14 +1,13 @@
 from __future__ import annotations
 
 from collections.abc import Collection, Iterator, Mapping, Sequence
-from functools import partial
 from typing import TYPE_CHECKING, Any, Literal, cast, overload
 
 import pyarrow as pa
 import pyarrow.compute as pc
 
 from narwhals._arrow.series import ArrowSeries
-from narwhals._arrow.utils import native_to_narwhals_dtype
+from narwhals._arrow.utils import concat_tables, native_to_narwhals_dtype, repeat
 from narwhals._compliant import EagerDataFrame
 from narwhals._expression_parsing import ExprKind
 from narwhals._utils import (
@@ -41,10 +40,10 @@ if TYPE_CHECKING:
     from narwhals._arrow.namespace import ArrowNamespace
     from narwhals._arrow.typing import (  # type: ignore[attr-defined]
         ChunkedArrayAny,
-        Mask,
         Order,
     )
     from narwhals._compliant.typing import CompliantDataFrameAny, CompliantLazyFrameAny
+    from narwhals._spark_like.utils import SparkSession
     from narwhals._translate import IntoArrowTable
     from narwhals._typing import _EagerAllowedImpl, _LazyAllowedImpl
     from narwhals._utils import Version, _LimitedContext
@@ -72,7 +71,6 @@ if TYPE_CHECKING:
         "right outer",
         "full outer",
     ]
-    PromoteOptions: TypeAlias = Literal["none", "default", "permissive"]
 
 
 class ArrowDataFrame(
@@ -128,6 +126,24 @@ class ArrowDataFrame(
             native = pa.Table.from_pydict(data, schema=pa_schema)
         return cls.from_native(native, context=context)
 
+    @classmethod
+    def from_dicts(
+        cls,
+        data: Sequence[Mapping[str, Any]],
+        /,
+        *,
+        context: _LimitedContext,
+        schema: IntoSchema | None,
+    ) -> Self:
+        from narwhals.schema import Schema
+
+        pa_schema = Schema(schema).to_arrow() if schema is not None else schema
+        if pa_schema and not data:
+            native = pa_schema.empty_table()
+        else:
+            native = pa.Table.from_pylist(data, schema=pa_schema)
+        return cls.from_native(native, context=context)
+
     @staticmethod
     def _is_native(obj: pa.Table | Any) -> TypeIs[pa.Table]:
         return isinstance(obj, pa.Table)
@@ -499,12 +515,9 @@ class ArrowDataFrame(
             row_index = (rank.over(partition_by=[], order_by=order_by) - 1).alias(name)
         return self.select(row_index, plx.all())
 
-    def filter(self, predicate: ArrowExpr | list[bool | None]) -> Self:
-        if isinstance(predicate, list):
-            mask_native: Mask | ChunkedArrayAny = predicate
-        else:
-            # `[0]` is safe as the predicate's expression only returns a single column
-            mask_native = self._evaluate_into_exprs(predicate)[0].native
+    def filter(self, predicate: ArrowExpr) -> Self:
+        # `[0]` is safe as the predicate's expression only returns a single column
+        mask_native = self._evaluate_into_exprs(predicate)[0].native
         return self._with_native(
             self.native.filter(mask_native), validate_column_names=False
         )
@@ -527,7 +540,12 @@ class ArrowDataFrame(
             )
         return self._with_native(df.slice(abs(n)), validate_column_names=False)
 
-    def lazy(self, backend: _LazyAllowedImpl | None = None) -> CompliantLazyFrameAny:
+    def lazy(
+        self,
+        backend: _LazyAllowedImpl | None = None,
+        *,
+        session: SparkSession | None = None,
+    ) -> CompliantLazyFrameAny:
         if backend is None:
             return self
         if backend is Implementation.DUCKDB:
@@ -535,9 +553,9 @@ class ArrowDataFrame(
 
             from narwhals._duckdb.dataframe import DuckDBLazyFrame
 
-            df = self.native  # noqa: F841
+            _df = self.native
             return DuckDBLazyFrame(
-                duckdb.table("df"), validate_backend_version=True, version=self._version
+                duckdb.table("_df"), validate_backend_version=True, version=self._version
             )
         if backend is Implementation.POLARS:
             import polars as pl  # ignore-banned-import
@@ -559,7 +577,7 @@ class ArrowDataFrame(
                 validate_backend_version=True,
                 version=self._version,
             )
-        if backend.is_ibis():
+        if backend is Implementation.IBIS:
             import ibis  # ignore-banned-import
 
             from narwhals._ibis.dataframe import IbisLazyFrame
@@ -570,6 +588,17 @@ class ArrowDataFrame(
                 version=self._version,
             )
 
+        if backend.is_spark_like():
+            from narwhals._spark_like.dataframe import SparkLikeLazyFrame
+
+            if session is None:
+                msg = "Spark like backends require `session` to be not None."
+                raise ValueError(msg)
+
+            return SparkLikeLazyFrame._from_compliant_dataframe(
+                self, session=session, implementation=backend, version=self._version
+            )
+
         raise AssertionError  # pragma: no cover
 
     def collect(
@@ -683,6 +712,7 @@ class ArrowDataFrame(
         *,
         keep: UniqueKeepStrategy,
         maintain_order: bool | None = None,
+        order_by: Sequence[str] | None,
     ) -> Self:
         # The param `maintain_order` is only here for compatibility with the Polars API
         # and has no effect on the output.
@@ -697,14 +727,30 @@ class ArrowDataFrame(
 
             agg_func = ArrowGroupBy._REMAP_UNIQUE[keep]
             col_token = generate_temporary_column_name(n_bytes=8, columns=self.columns)
+            if order_by and maintain_order:
+                idx_token = generate_temporary_column_name(
+                    n_bytes=8, columns=[*self.columns, col_token]
+                )
+                df = (
+                    self.with_row_index(idx_token, order_by=None)
+                    .sort(*order_by, nulls_last=False, descending=False)
+                    .unique(subset=subset, keep=keep, maintain_order=False, order_by=None)
+                )
+                return df.sort(idx_token, descending=False, nulls_last=False).drop(
+                    [idx_token], strict=False
+                )
+            if order_by:
+                native = self.sort(*order_by, nulls_last=False, descending=False).native
+            else:
+                native = self.native
             keep_idx_native = (
-                self.native.append_column(col_token, pa.array(np.arange(len(self))))
+                native.append_column(col_token, pa.array(np.arange(len(self))))
                 .group_by(subset)
                 .aggregate([(col_token, agg_func)])
                 .column(f"{col_token}_{agg_func}")
             )
             return self._with_native(
-                self.native.take(keep_idx_native), validate_column_names=False
+                native.take(keep_idx_native), validate_column_names=False
             )
 
         keep_idx = self.simple_select(*subset).is_unique()
@@ -742,34 +788,19 @@ class ArrowDataFrame(
         variable_name: str,
         value_name: str,
     ) -> Self:
-        n_rows = len(self)
-        index_ = [] if index is None else index
-        on_ = [c for c in self.columns if c not in index_] if on is None else on
-        concat = (
-            partial(pa.concat_tables, promote_options="permissive")
-            if self._backend_version >= (14, 0, 0)
-            else pa.concat_tables
-        )
-        names = [*index_, variable_name, value_name]
-        return self._with_native(
-            concat(
-                [
-                    pa.Table.from_arrays(
-                        [
-                            *(self.native.column(idx_col) for idx_col in index_),
-                            cast(
-                                "ChunkedArrayAny",
-                                pa.array([on_col] * n_rows, pa.string()),
-                            ),
-                            self.native.column(on_col),
-                        ],
-                        names=names,
-                    )
-                    for on_col in on_
-                ]
-            )
-        )
         # TODO(Unassigned): Even with promote_options="permissive", pyarrow does not
         # upcast numeric to non-numeric (e.g. string) datatypes
+        n = len(self)
+        index = [] if index is None else list(index)
+        on_ = (c for c in self.columns if c not in index) if on is None else iter(on)
+        index_cols = self.native.select(index)
+        column = self.native.column
+        tables = (
+            index_cols.append_column(variable_name, repeat(name, n)).append_column(
+                value_name, column(name)
+            )
+            for name in on_
+        )
+        return self._with_native(concat_tables(tables, "permissive"))
 
     pivot = not_implemented()
